package de.fd.fh.server.access;

import de.fd.fh.server.user.events.ChangePasswordEvent;
import de.fd.fh.server.user.events.ChangeUserNameEvent;
import lombok.RequiredArgsConstructor;

import java.util.Observable;
import java.util.Observer;

@RequiredArgsConstructor
public class AccessContextEventListener implements Observer
{
    private final AccessRepository accessRepository;

    @Override
    public void update(Observable observable, Object event)
    {
        if (event instanceof ChangePasswordEvent)
        {
            handlePasswordChange((ChangePasswordEvent) event);
        }
        else if (event instanceof ChangeUserNameEvent)
        {
            handleRename((ChangeUserNameEvent) event);
        }
    }

    public void handleRename(ChangeUserNameEvent event)
    {
        final Access access = accessRepository.findByUserId(event.getUserId());

        access.rename(event.getNewName());

        accessRepository.save(access);
    }

    public void handlePasswordChange(ChangePasswordEvent event)
    {
        final Access access = accessRepository.findByUserId(event.getUserId());

        access.updatePassword(event.getNewPassword());

        accessRepository.save(access);
    }
}