package de.fd.fh.server.access.web;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.fd.fh.server.access.AccessService;
import de.fd.fh.server.access.AccessToken;
import de.fd.fh.server.user.UserId;
import de.fd.fh.shared.Utils;
import de.fd.fh.shared.network.messages.LoginResponse;
import de.fd.fh.shared.network.messages.RegistrateRequest;
import lombok.RequiredArgsConstructor;
import spark.Request;
import spark.Response;

@RequiredArgsConstructor
public class AccessController
{
    private final ObjectMapper objectMapper = new ObjectMapper();
    private final AccessService service;

    public Response deletePlayer(Request request, Response response)
    {
        final UserId userId = UserId.of(request.params(":player_id"));
        final AccessToken token = request.session().attribute("userId");

        if (service.deleteAccount(userId, token))
        {
            response.status(200);
        }
        else
        {
            response.status(400);
        }

        return response;
    }

    public Object logout(Request request, Response response)
    {
        final String token = request.headers(Utils.AUTHENTICATION_HEADER);

        if (service.logout(token))
        {
            response.status(200);
        }
        else
        {
            response.status(400);
        }

        return response;
    }

    public Object login(Request request, Response response) throws JsonProcessingException
    {
        final String header = request.headers(Utils.AUTHENTICATION_HEADER);

        final LoginResponse login = service.authorization(header);

        if (login == null)
        {
            response.status(401);
        }
        else
        {
            response.status(200);
            response.type("application/json");
            response.body(objectMapper.writeValueAsString(login));
        }
        return response;
    }

    public Object registrate(Request request, Response response) throws java.io.IOException
    {
        final RegistrateRequest message =
                objectMapper.readValue(request.body(), RegistrateRequest.class);

        if (service.createPlayer(message))
        {
            response.status(201);
        }
        else
        {
            response.status(400);
        }
        return response;
    }

    public void before(final Request req)
    {
        final String path = req.pathInfo();
        final String token = req.headers(Utils.AUTHENTICATION_HEADER);

        final AccessToken accessToken = service.before(path, token);

        req.session().attribute("userId",
                accessToken);
    }
}
