package de.fd.fh.server.access;

import de.fd.fh.server.user.UserId;
import de.fd.fh.server.user.events.ChangePasswordEvent;
import de.fd.fh.server.user.events.ChangeUserNameEvent;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.BDDMockito.then;
import static org.mockito.Mockito.*;

class AccessContextEventListenerTest
{
    @Test
    void given_changePasswordEvent_when_passwordChanged_should_changePassword()
    {
        final ChangePasswordEvent event = new ChangePasswordEvent(UserId.of("12345"), "newPwd");

        final AccessRepository repository = mock(AccessRepository.class);
        when(repository.findByUserId(any(UserId.class)))
                .thenReturn(new Access());

        final ArgumentCaptor<Access> captor = ArgumentCaptor.forClass(Access.class);

        new de.fd.fh.server.access.AccessContextEventListener(repository).update(null, event);
        verify(repository).save(captor.capture());

        assertEquals("newPwd", captor.getValue().getPassword(), "Have to be the new password");
        then(repository).should().findByUserId(any(UserId.class));
        then(repository).should().save(any(Access.class));
        then(repository).shouldHaveNoMoreInteractions();
    }

    @Test
    void given_changeNameEvent_when_nameChanged_should_renameAccess()
    {
        final ChangeUserNameEvent event = new ChangeUserNameEvent(UserId.of("12345"), "newName");

        final AccessRepository repository = mock(AccessRepository.class);
        when(repository.findByUserId(any(UserId.class)))
                .thenReturn(new Access());

        final ArgumentCaptor<Access> captor = ArgumentCaptor.forClass(Access.class);

        new de.fd.fh.server.access.AccessContextEventListener(repository).update(null, event);
        verify(repository).save(captor.capture());

        assertEquals("newName", captor.getValue().getName(), "Have to be the new name");
        then(repository).should().findByUserId(any(UserId.class));
        then(repository).should().save(any(Access.class));
        then(repository).shouldHaveNoMoreInteractions();
    }
}